<?php
/*
 * This file is part of Sylar.
 *
 * Sylar is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Sylar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Sylar.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @copyright Copyright Sylar Development Team
 * @license http://www.gnu.org/licenses/ GNU Public License V2.0
 * @see https://launchpad.net/sylar/
 * @see http://www.giano-solutions.com
 */
 

import('sylar.presentation.Format');


/**
 * Main Html Format Class
 * 
 * This class is extended from various Html Sylar Classes and collect 
 * common method uses to formato HTML source
 * 
 * @package Sylar
 * @version 1.0
 * @since 03/apr/08
 * @author Gianluca Giusti [brdp] <g.giusti@giano-solutions.com>
 * @copyright Sylar Development Team
 */
 
class Sylar_Html extends Sylar_Format{
	
	// Tag Attributes
	private $aAttributes;
	
	// Tag Attributes without names for example checked, selected, disabled, etc...
	private $aNoNameAttributes;
	
	function __construct(){
		$this->aAttributes = array();
		$this->aNoNameAttributes = array();
	}
	function __destruct(){
		// nothing to do at the moment
	}
	
	
	
	//								 							  Public Methods
	//__________________________________________________________________________
		
	
	/**
	 * Add an attribute name and value into the tag attribute list
	 * 
	 * @return void
	 * @param string $sAttributeName name
	 * @param string $sAttributeValue value
	 * 
	 * @see $this->aAttributes
	 */
	public function setAttribute($sAttributeName, $sAttributeValue){
		$this->aAttributes[$sAttributeName] = $sAttributeValue;
	}


	/**
	 * Add a NoName attribute value into the tag attribute list. 
	 * Fore example DISABLED, CHECKED, etc...
	 * 
	 * @return void
	 * @param string $sAttributeValue value
	 * 
	 * @see $this->aNoNameAttributes
	 */
	public function setAttributeWithoutName($sAttributeValue){
		$this->aNoNameAttributes[$sAttributeValue] = $sAttributeValue;
	}	
	

	/**
	 * Return attribute value from the internal list. Null if doesn't exists
	 * 
	 * @return void
	 * @param string $sAttributeName name
	 * 
	 * @see $this->aAttributes
	 */	
	public function getAttribute($sAttributeName){
		if(array_key_exists($sAttributeName, $this->aAttributes)){
			return $this->aAttributes[$sAttributeName];
		}
		return null;
	}	

	
	/**
	 * Return attribute value from the internal list. Null if doesn't exists
	 * Used for attributes like CHECKED, SELECTED, etc...
	 * 
	 * @return void
	 * @param string $sAttributeValue value
	 * 
	 * @see $this->aAttributes
	 */	
	public function getAttributeWithoutName($sAttributeValue){
		if(array_key_exists($sAttributeValue, $this->aNoNameAttributes)){
			return $this->aNoNameAttributes[$sAttributeValue];
		}
		return null;
	}
		
	
	/**
	 * Remove an attribute from the internal list. 
	 * 
	 * @param string $sAttributeName name
	 * 
	 * @see $this->aAttributes
	 */	
	public function removeAttribute($sAttributeName){
		if(array_key_exists($sAttributeName, $this->aAttributes)){
			unset($this->aAttributes[$sAttributeName]);
		}
	}	
	

	/**
	 * Remove an attribute from the internal list. 
	 * Used for attributes like CHECKED, SELECTED, etc...
	 * 
	 * @param string $sAttributeValue value
	 * 
	 * @see $this->aNoNameAttributes
	 */	
	public function removeAttributeWithoutName($sAttributeValue){
		if(array_key_exists($sAttributeValue, $this->aNoNameAttributes)){
			unset($this->aNoNameAttributes[$sAttributeValue]);
		}
	}	
	

	/**
	 * Return the Tag HTML code with all attribute contained into the internal list
	 * 
	 * @return void
	 * @param string $sTag Tag name for example TABLE, DIV, etc...
	 * 
	 * @see $this->aAttributes
	 */	
	public function provideTagWithAttributes($sTag){
		return Sylar_Html::fillTagAttributes($sTag, $this->aAttributes, $this->aNoNameAttributes);
	}
	
	
	/**
	 * Fill Html Attributes in Tag
	 * It generate the HTML tag $sTagStart with all attributes secified in the array $aAttributes
	 * The array is formatted like Key=>Value, Key is the attribute name and Value is the value of attribute. 
	 * It ignores all null data contained in the attributes array. 
	 * 
	 * For example:
	 * $sTagStart = "div";
	 * $aAttributes = array("id"=>"My_ID", "class"=>"My_CSS_Class");
	 * 
	 * calling the method:
	 * fillTagAttributes($sTagStart, $aAttributes)
	 * it returns a string thath contain: <div id="My_ID" class="My_CSS_Class">
	 * 
	 * @return string
	 * @param string $sTagStart The begin of the tag for example: script or div without '<'
	 * @param array $aAttributes An array Key=>Value to insert as attributes in the Tag.
	 * @param array $aNoNameAttributes An array Key=>Value to insert as attributes without name in the Tag, like DISABLED, CHECKED, SELECTED, etc...
	 */
	public static function fillTagAttributes($sTagStart, $aAttributes, $aNoNameAttributes=null){
		
		if(is_null($sTagStart) || strlen(str_replace(" ", "",$sTagStart))==0){
			throw new ExceptionInSylar("HTML TAG is not declared. [Sylar_Html::fillTagAttributes]");
			return null;
		}
		
		
		$sTagStart = "<".$sTagStart;
		
		// Attribute Tag with NAME
		if(!is_null($aAttributes) && is_array($aAttributes)){
			foreach ($aAttributes as $k => $v) {
				if(!is_null($v)){
					$sTagStart .= " ".$k."=\"".$v."\"";
				}
			}
		}
		
		// Attribute Tag without NAME
		if(!is_null($aNoNameAttributes) && is_array($aNoNameAttributes)){
			foreach ($aNoNameAttributes as $k => $v) {
				if(!is_null($v)){
					$sTagStart .= " {$v}";
				}
			}
		}
		
		
		$sTagStart .= ">";
		
		return $sTagStart;
	}
	
	
	
	
	
	
	
	//								 							 Private Methods
	//__________________________________________________________________________
	
	
	
}


?>